@@ -3,8 +3,9 @@ package DateTime::Format::Natural::Lang::EN;
 use strict;
 use warnings;
 use base qw(DateTime::Format::Natural::Lang::Base);
+use boolean qw(true false);
 
-our $VERSION = '1.13';
+our $VERSION = '1.15';
 
 our (%init,
      %timespan,
@@ -22,10 +23,10 @@ our (%init,
 
 %RE = ('number'    => qr/^(\d+)$/,
        'year'      => qr/^(\d{4})$/,
-       'time'      => qr/^((?:\d{1,2})(?:\:\d{1,2})?)$/,
-       'time_am'   => qr/^((?:\d+)(?:\:\d+)?)(?:am)?$/,
-       'time_pm'   => qr/^((?:\d+)(?:\:\d+)?)pm$/,
-       'time_full' => qr/^(\d{2}\:\d{2}:\d{2})$/,
+       'time'      => qr/^((?:\d{1,2})(?:\:\d{2})?)$/,
+       'time_am'   => qr/^((?:\d{1,2})(?:\:\d{2})?)(?:am)?$/,
+       'time_pm'   => qr/^((?:\d{1,2})(?:\:\d{2})?)pm$/,
+       'time_full' => qr/^(\d{1,2}\:\d{2}\:\d{2})$/,
        'day_enum'  => qr/^(\d+)(?:st|nd|rd|th)?$/,
        'monthday'  => qr/^(\d{1,2})$/);
 {
@@ -43,6 +44,9 @@ our (%init,
     my $days_re = join '|', @data_weekdays_all;
     $RE{weekday} = qr/^($days_re)$/i;
 
+    $days_re = join '|', map "${_}s?", @data_weekdays_all;
+    $RE{weekdays} = qr/^($days_re)$/i;
+
     $i = 1;
 
     %data_months = map {
@@ -62,7 +66,8 @@ our (%init,
 # <keyword> => [
 #    [ <PERL TYPE DECLARATION>, ... ], ---------------------> declares how the tokens will be evaluated
 #    [
-#      { <token index> => <token value>, ... }, ------------> declares the index <-> value map 
+#      { <token index> => <token value>, ... }, ------------> declares the index <-> value map
+#      [ <subroutine(s) for extended check(s)>, ... ] ------> declares the extended check(s)
 #      [ [ <index(es) of token(s) to be passed> ], ... ], --> declares which tokens will be passed to the dispatch handler(s)
 #      [ <name of subroutine to dispatch to>, ... ], -------> declares the dispatch handler(s)
 #    ],
@@ -72,6 +77,7 @@ our (%init,
        [ 'SCALAR' ],
        [
          { 0 => 'now' },
+         [],
          [ [] ],
          [ '_day_today' ],
        ],
@@ -80,16 +86,19 @@ our (%init,
        [ 'SCALAR' ],
        [
          { 0 => 'today' },
+         [],
          [ [] ],
          [ '_day_today' ],
        ],
        [
          { 0 => 'yesterday' },
+         [],
          [ [] ],
          [ '_day_yesterday' ],
        ],
        [
          { 0 => 'tomorrow' },
+         [],
          [ [] ],
          [ '_day_tomorrow' ],
        ],
@@ -98,16 +107,19 @@ our (%init,
        [ 'SCALAR' ],
        [
          { 0 => 'morning' },
+         [],
          [ [] ],
          [ '_daytime_morning' ],
        ],
        [
          { 0 => 'afternoon' },
+         [],
          [ [] ],
          [ '_daytime_afternoon' ],
        ],
        [
          { 0 => 'evening' },
+         [],
          [ [] ],
          [ '_daytime_evening' ],
        ]
@@ -116,11 +128,13 @@ our (%init,
        [ 'SCALAR' ],
        [
          { 0 => 'noon' },
+         [],
          [ [] ],
          [ '_daytime_noon' ],
        ],
        [
          { 0 => 'midnight' },
+         [],
          [ [] ],
          [ '_daytime_midnight' ],
        ],
@@ -129,31 +143,37 @@ our (%init,
        [ 'SCALAR', 'SCALAR', 'SCALAR' ],
        [
          { 0 => 'yesterday', 1 => 'at', 2 => 'noon' },
+         [],
          [ [], [] ],
          [ '_day_yesterday', '_daytime_noon' ],
        ],
        [
          { 0 => 'yesterday', 1 => 'at', 2 => 'midnight' },
+         [],
          [ [], [] ],
          [ '_day_yesterday', '_daytime_midnight' ],
        ],
        [
          { 0 => 'today', 1 => 'at', 2 => 'noon' },
+         [],
          [ [], [] ],
          [ '_day_today', '_daytime_noon' ],
        ],
        [
          { 0 => 'today', 1 => 'at', 2 => 'midnight' },
+         [],
          [ [], [] ],
          [ '_day_today', '_daytime_midnight' ],
        ],
        [
          { 0 => 'tomorrow', 1 => 'at', 2 => 'noon' },
+         [],
          [ [], [] ],
          [ '_day_tomorrow', '_daytime_noon' ],
        ],
        [
          { 0 => 'tomorrow', 1 => 'at', 2 => 'midnight' },
+         [],
          [ [], [] ],
          [ '_day_tomorrow', '_daytime_midnight' ],
        ],
@@ -162,16 +182,19 @@ our (%init,
        [ 'SCALAR', 'SCALAR' ],
        [
          { 0 => 'this', 1 => 'morning' },
+         [],
          [ [] ],
          [ '_daytime_morning' ],
        ],
        [
          { 0 => 'this', 1 => 'afternoon' },
+         [],
          [ [] ],
          [ '_daytime_afternoon' ],
        ],
        [
          { 0 => 'this', 1 => 'evening' },
+         [],
          [ [] ],
          [ '_daytime_evening' ],
        ],
@@ -180,46 +203,55 @@ our (%init,
        [ 'SCALAR', 'SCALAR' ],
        [
          { 0 => 'yesterday', 1 => 'morning' },
+         [],
          [ [], [] ],
          [ '_day_yesterday', '_daytime_morning' ],
        ],
        [
          { 0 => 'yesterday', 1 => 'afternoon' },
+         [],
          [ [], [] ],
          [ '_day_yesterday', '_daytime_afternoon' ],
        ],
        [
          { 0 => 'yesterday', 1 => 'evening' },
+         [],
          [ [], [] ],
          [ '_day_yesterday', '_daytime_evening' ],
        ],
        [
          { 0 => 'today', 1 => 'morning' },
+         [],
          [ [], [] ],
          [ '_day_today', '_daytime_morning' ],
        ],
        [
          { 0 => 'today', 1 => 'afternoon' },
+         [],
          [ [], [] ],
          [ '_day_today', '_daytime_afternoon' ],
        ],
        [
          { 0 => 'today', 1 => 'evening' },
+         [],
          [ [], [] ],
          [ '_day_today', '_daytime_evening' ],
        ],
        [
          { 0 => 'tomorrow', 1 => 'morning' },
+         [],
          [ [], [] ],
          [ '_day_tomorrow', '_daytime_morning' ],
        ],
        [
          { 0 => 'tomorrow', 1 => 'afternoon' },
+         [],
          [ [], [] ],
          [ '_day_tomorrow', '_daytime_afternoon' ],
        ],
        [
          { 0 => 'tomorrow', 1 => 'evening' },
+         [],
          [ [], [] ],
          [ '_day_tomorrow', '_daytime_evening' ],
        ]
@@ -228,31 +260,37 @@ our (%init,
        [ 'REGEXP', 'SCALAR' ],
        [
          { 0 => $RE{time_am}, 1 => 'yesterday' },
+         [],
          [ [ 0 ], [] ],
          [ '_time', '_day_yesterday' ],
        ],
        [
          { 0 => $RE{time_am}, 1 => 'today' },
+         [],
          [ [ 0 ], [] ],
          [ '_time', '_day_today' ],
        ],
        [
          { 0 => $RE{time_am}, 1 => 'tomorrow' },
+         [],
          [ [ 0 ], [] ],
          [ '_time', '_day_tomorrow' ],
        ],
        [
          { 0 => $RE{time_pm}, 1 => 'yesterday' },
+         [],
          [ [ 0 ], [] ],
          [ '_at_pm', '_day_yesterday' ],
        ],
        [
          { 0 => $RE{time_pm}, 1 => 'today' },
+         [],
          [ [ 0 ], [] ],
          [ '_at_pm', '_day_today' ],
        ],
        [
          { 0 => $RE{time_pm}, 1 => 'tomorrow' },
+         [],
          [ [ 0 ], [] ],
          [ '_at_pm', '_day_tomorrow' ],
        ],
@@ -261,6 +299,7 @@ our (%init,
        [ 'REGEXP' ],
        [
          { 0 => $RE{month} },
+         [],
          [ [ 0 ] ],
          [ '_month' ],
        ],
@@ -269,11 +308,13 @@ our (%init,
        [ 'REGEXP', 'REGEXP' ],
        [
          { 0 => $RE{day_enum}, 1 => $RE{month} },
+         [],
          [ [ 0, 1 ] ],
          [ '_month_day_before' ],
        ],
        [
          { 0 => $RE{month}, 1 => $RE{day_enum} },
+         [],
          [ [ 0, 1 ] ],
          [ '_month_day_after' ],
        ]
@@ -282,6 +323,7 @@ our (%init,
        [ 'REGEXP' ],
        [
          { 0 => $RE{weekday} },
+         [],
          [ [ 0 ] ],
          [ '_weekday' ],
        ],
@@ -290,6 +332,7 @@ our (%init,
        [ 'SCALAR', 'REGEXP' ],
        [
          { 0 => 'last', 1 => $RE{weekday} },
+         [],
          [ [ 1 ] ],
          [ '_last_day' ],
        ],
@@ -298,6 +341,7 @@ our (%init,
        [ 'SCALAR', 'REGEXP' ],
        [
          { 0 => 'last', 1 => $RE{month} },
+         [],
          [ [ 1 ] ],
          [ '_last_month' ],
        ],
@@ -306,6 +350,7 @@ our (%init,
        [ 'SCALAR', 'SCALAR' ],
        [
          { 0 => 'last', 1 => 'month' },
+         [],
          [ [] ],
          [ '_last_month_literal' ],
        ],
@@ -314,6 +359,7 @@ our (%init,
        [ 'SCALAR', 'SCALAR' ],
        [
          { 0 => 'last', 1 => 'year' },
+         [],
          [ [] ],
          [ '_last_year' ],
        ],
@@ -322,6 +368,7 @@ our (%init,
        [ 'SCALAR', 'REGEXP' ],
        [
          { 0 => 'next', 1 => $RE{weekday} },
+         [],
          [ [ 1 ] ],
          [ '_next_weekday' ],
        ],
@@ -330,6 +377,7 @@ our (%init,
        [ 'SCALAR', 'REGEXP' ],
        [
          { 0 => 'next', 1 => $RE{month} },
+         [],
          [ [ 1 ] ],
          [ '_next_month' ],
        ],
@@ -338,6 +386,7 @@ our (%init,
        [ 'SCALAR', 'SCALAR' ],
        [
          { 0 => 'next', 1 => 'month' },
+         [],
          [ [] ],
          [ '_next_month_literal' ],
        ],
@@ -346,6 +395,7 @@ our (%init,
        [ 'SCALAR', 'SCALAR' ],
        [
          { 0 => 'next', 1 => 'year' },
+         [],
          [ [] ],
          [ '_next_year' ],
        ],
@@ -354,6 +404,7 @@ our (%init,
        [ 'SCALAR', 'SCALAR' ],
        [
          { 0 => 'this', 1 => 'second' },
+         [],
          [ [] ],
          [ '_this_second' ],
        ],
@@ -362,6 +413,7 @@ our (%init,
        [ 'SCALAR', 'REGEXP' ],
        [
          { 0 => 'this', 1 => $RE{weekday} },
+         [],
          [ [ 1 ] ],
          [ '_this_weekday' ],
        ],
@@ -370,6 +422,7 @@ our (%init,
        [ 'SCALAR', 'SCALAR' ],
        [
          { 0 => 'this', 1 => 'month' },
+         [],
          [ [] ],
          [ '_this_month' ],
        ],
@@ -378,24 +431,28 @@ our (%init,
        [ 'REGEXP', 'SCALAR' ],
        [
          { 0 => $RE{time}, 1 => 'am' },
+         [],
          [ [ 0 ] ],
          [ '_at_am' ],
        ],
        [
          { 0 => $RE{time}, 1 => 'pm' },
-         [ [ 0 ] ], 
+         [],
+         [ [ 0 ] ],
          [ '_at_pm' ],
        ],
     ],
     at_combined => [
        [ 'REGEXP' ],
        [
-         { 0 => qr/^(\d+)(?:am)$/i }, 
+         { 0 => $RE{time_am} },
+         [],
          [ [ 0 ] ],
          [ '_at_am' ],
        ],
        [
-         { 0 => qr/^(\d+)(?:pm)$/i },
+         { 0 => $RE{time_pm} },
+         [],
          [ [ 0 ] ],
          [ '_at_pm' ],
        ],
@@ -404,6 +461,7 @@ our (%init,
        [ 'REGEXP', 'REGEXP' ],
        [
          { 0 => $RE{weekday}, 1 => $RE{time} },
+         [],
          [ [ 0 ], [ 1 ] ],
          [ '_weekday', '_time' ],
        ],
@@ -412,6 +470,7 @@ our (%init,
        [ 'REGEXP' ],
        [
          { 0 => $RE{time} },
+         [],
          [ [ 0 ] ],
          [ '_time' ],
        ],
@@ -420,6 +479,7 @@ our (%init,
        [ 'REGEXP' ],
        [
          { 0 => $RE{time_full} },
+         [],
          [ [ 0 ] ],
          [ '_time_full' ],
        ],
@@ -428,6 +488,7 @@ our (%init,
        [ 'REGEXP', 'REGEXP' ],
        [
          { 0 => $RE{month}, 1 => $RE{year} },
+         [],
          [ [ 0 ], [ 1 ] ],
          [ '_month', '_year' ],
        ],
@@ -436,6 +497,7 @@ our (%init,
        [ 'REGEXP' ],
        [
          { 0 => $RE{year} },
+         [],
          [ [ 0 ] ],
          [ '_year' ],
        ],
@@ -444,6 +506,7 @@ our (%init,
        [ 'REGEXP', 'REGEXP' ],
        [
          { 0 => $RE{day_enum}, 1 => $RE{weekday} },
+         [],
          [ [ 0, 1 ] ],
          [ '_count_weekday' ],
        ],
@@ -452,6 +515,7 @@ our (%init,
        [ 'REGEXP', 'SCALAR' ],
        [
          { 0 => $RE{day_enum}, 1 => 'day' },
+         [],
          [ [ 0 ] ],
          [ '_count_yearday' ],
        ],
@@ -460,16 +524,19 @@ our (%init,
        [ 'REGEXP', 'SCALAR', 'SCALAR', 'SCALAR' ],
        [
          { 0 => $RE{number}, 1 => 'in', 2 => 'the', 3 => 'morning' },
+         [],
          [ [ 0 ] ],
          [ '_daytime_in_the_morning' ],
        ],
        [
          { 0 => $RE{number}, 1 => 'in', 2 => 'the', 3 => 'afternoon' },
+         [],
          [ [ 0 ] ],
          [ '_daytime_in_the_afternoon' ],
        ],
        [
          { 0 => $RE{number}, 1 => 'in', 2 => 'the', 3 => 'evening' },
+         [],
          [ [ 0 ] ],
          [ '_daytime_in_the_evening' ],
        ],
@@ -478,36 +545,43 @@ our (%init,
        [ 'REGEXP', 'REGEXP', 'SCALAR' ],
        [
          { 0 => $RE{number}, 1 => qr/seconds?/i, 2 => 'ago' },
+         [],
          [ [ 0 ] ],
          [ '_ago_seconds' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/minutes?/i, 2 => 'ago' },
+         [],
          [ [ 0 ] ],
          [ '_ago_minutes' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'ago' },
+         [],
          [ [ 0 ] ],
          [ '_ago_hours' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/days?/i, 2 => 'ago' },
+         [],
          [ [ 0 ] ],
          [ '_ago_days' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/weeks?/i, 2 => 'ago' },
+         [],
          [ [ 0 ] ],
          [ '_ago_weeks' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/months?/i, 2 => 'ago' },
+         [],
          [ [ 0 ] ],
          [ '_ago_months' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/years?/i, 2 => 'ago' },
+         [],
          [ [ 0 ] ],
          [ '_ago_years' ],
        ],
@@ -516,36 +590,43 @@ our (%init,
        [ 'SCALAR', 'REGEXP', 'REGEXP', 'SCALAR' ],
        [
          { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/seconds?/i, 3 => 'ago' },
+         [],
          [ [], [ 1 ] ],
          [ '_day_tomorrow', '_ago_seconds' ],
        ],
        [
          { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/minutes?/i, 3 => 'ago' },
+         [],
          [ [], [ 1 ] ],
          [ '_day_tomorrow', '_ago_minutes' ],
        ],
        [
          { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/hours?/i, 3 => 'ago' },
+         [],
          [ [], [ 1 ] ],
          [ '_day_tomorrow', '_ago_hours' ],
        ],
        [
          { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/days?/i, 3 => 'ago' },
+         [],
          [ [], [ 1 ] ],
          [ '_day_tomorrow', '_ago_days' ],
        ],
        [
          { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/weeks?/i, 3 => 'ago' },
+         [],
          [ [], [ 1 ] ],
          [ '_day_tomorrow', '_ago_weeks' ],
        ],
        [
          { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/months?/i, 3 => 'ago' },
+         [],
          [ [], [ 1 ] ],
          [ '_day_tomorrow', '_ago_months' ],
        ],
        [
          { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/years?/i, 3 => 'ago' },
+         [],
          [ [], [ 1 ] ],
          [ '_day_tomorrow', '_ago_years' ],
        ],
@@ -554,36 +635,43 @@ our (%init,
        [ 'REGEXP', 'REGEXP', 'REGEXP', 'SCALAR' ],
        [
          { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/seconds?/i, 3 => 'ago' },
+         [],
          [ [], [ 1 ] ],
          [ '_day_yesterday', '_ago_seconds' ],
        ],
        [
          { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/minutes?/i, 3 => 'ago' },
+         [],
          [ [], [ 1 ] ],
          [ '_day_yesterday', '_ago_minutes' ],
        ],
        [
          { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/hours?/i, 3 => 'ago' },
+         [],
          [ [], [ 1 ] ],
          [ '_day_yesterday', '_ago_hours' ],
        ],
        [
          { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/days?/i, 3 => 'ago' },
+         [],
          [ [], [ 1 ] ],
          [ '_day_yesterday', '_ago_days' ],
        ],
        [
          { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/weeks?/i, 3 => 'ago' },
+         [],
          [ [], [ 1 ] ],
          [ '_day_yesterday', '_ago_weeks' ],
        ],
        [
          { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/months?/i, 3 => 'ago' },
+         [],
          [ [], [ 1 ] ],
          [ '_day_yesterday', '_ago_months' ],
        ],
        [
          { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/years?/i, 3 => 'ago' },
+         [],
          [ [], [ 1 ] ],
          [ '_day_yesterday', '_ago_years' ],
        ],
@@ -592,11 +680,13 @@ our (%init,
        [ 'REGEXP', 'REGEXP', 'REGEXP', 'SCALAR', 'SCALAR', 'REGEXP' ],
        [
          { 0 => $RE{weekday}, 1 => $RE{number}, 2 => qr/months?/, 3 => 'ago', 4 => 'at', 5 => $RE{time_am} },
+         [],
          [ [ 1 ], [ 0 ], [ 5 ] ],
          [ '_ago_months', '_weekday', '_time' ],
        ],
        [
          { 0 => $RE{weekday}, 1 => $RE{number}, 2 => qr/months?/, 3 => 'ago', 4 => 'at', 5 => $RE{time_pm} },
+         [],
          [ [ 1 ], [ 0 ], [ 5 ] ],
          [ '_ago_months', '_weekday', '_at_pm' ],
        ],
@@ -605,61 +695,73 @@ our (%init,
        [ 'REGEXP', 'REGEXP', 'SCALAR', 'SCALAR' ],
        [
          { 0 => $RE{number}, 1 => qr/minutes?/i, 2 => 'before', 3 => 'now' },
+         [],
          [ [ 0 ] ],
          [ '_now_minutes_before' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/minutes?/i, 2 => 'from', 3 => 'now' },
+         [],
          [ [ 0 ] ],
          [ '_now_minutes_from' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'before', 3 => 'now' },
+         [],
          [ [ 0 ] ],
          [ '_now_hours_before' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'from', 3 => 'now' },
+         [],
          [ [ 0 ] ],
          [ '_now_hours_from' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/days?/i,  2 => 'before', 3 => 'now' },
+         { 0 => $RE{number}, 1 => qr/^days?/i,  2 => 'before', 3 => 'now' },
+         [],
          [ [ 0 ] ],
          [ '_now_days_before' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/days?/i, 2 => 'from', 3 => 'now' },
+         { 0 => $RE{number}, 1 => qr/^days?/i, 2 => 'from', 3 => 'now' },
+         [],
          [ [ 0 ] ],
          [ '_now_days_from' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/weeks?/i, 2 => 'before', 3 => 'now' },
+         [],
          [ [ 0 ] ],
          [ '_now_weeks_before' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/weeks?/i, 2 => 'from', 3 => 'now' },
+         [],
          [ [ 0 ] ],
          [ '_now_weeks_from' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/months?/i, 2 => 'before', 3 => 'now' },
+         [],
          [ [ 0 ] ],
          [ '_now_months_before' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/months?/i, 2 => 'from', 3 => 'now' },
+         [],
          [ [ 0 ] ],
          [ '_now_months_from' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/years?/i, 2 => 'before', 3 => 'now' },
+         [],
          [ [ 0 ] ],
          [ '_now_years_before' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/years?/i, 2 => 'from', 3 => 'now' },
+         [],
          [ [ 0 ] ],
          [ '_now_years_from' ],
        ],
@@ -668,16 +770,19 @@ our (%init,
        [ 'REGEXP', 'REGEXP', 'SCALAR', 'SCALAR', 'SCALAR' ],
        [
          { 0 => $RE{weekday}, 1 => $RE{number}, 2 => 'in', 3 => 'the', 4 => 'morning' },
+         [],
          [ [ 0 ], [ 1 ] ],
          [ '_weekday', '_daytime_in_the_morning' ],
        ],
        [
          { 0 => $RE{weekday}, 1 => $RE{number}, 2 => 'in', 3 => 'the', 4 => 'afternoon' },
+         [],
          [ [ 0 ], [ 1 ] ],
          [ '_weekday', '_daytime_in_the_afternoon' ],
        ],
        [
          { 0 => $RE{weekday}, 1 => $RE{number}, 2 => 'in', 3 => 'the', 4 => 'evening' },
+         [],
          [ [ 0 ], [ 1 ] ],
          [ '_weekday', '_daytime_in_the_evening' ],
        ],
@@ -687,6 +792,7 @@ our (%init,
        [ 'SCALAR', 'REGEXP', 'SCALAR', 'REGEXP' ],
        [
          { 0 => 'last', 1 => $RE{weekday}, 2 => 'at', 3 => $RE{time} },
+         [],
          [ [ 1 ], [ 3 ] ],
          [ '_last_day', '_time' ],
        ],
@@ -695,6 +801,7 @@ our (%init,
        [ 'SCALAR', 'SCALAR', 'REGEXP' ],
        [
          { 0 => 'last', 1 => 'week', 2 => $RE{weekday} },
+         [],
          [ [ 2 ] ],
          [ '_last_week_day' ],
        ],
@@ -703,6 +810,7 @@ our (%init,
        [ 'REGEXP', 'SCALAR', 'SCALAR' ],
        [
          { 0 => $RE{weekday}, 1 => 'last', 2 => 'week' },
+         [],
          [ [ 0 ] ],
          [ '_day_last_week' ],
        ],
@@ -711,6 +819,7 @@ our (%init,
        [ 'REGEXP', 'SCALAR', 'SCALAR', 'SCALAR' ],
        [
          { 0 => $RE{day_enum}, 1 => 'day', 2 => 'last', 3 => 'week' },
+         [],
          [ [ 0 ] ],
          [ '_count_day_last_week' ],
        ],
@@ -719,6 +828,7 @@ our (%init,
        [ 'REGEXP', 'SCALAR', 'SCALAR', 'SCALAR' ],
        [
          { 0 => $RE{day_enum}, 1 => 'day', 2 => 'last', 3 => 'month' },
+         [],
          [ [ 0 ] ],
          [ '_count_day_last_month' ],
        ],
@@ -728,6 +838,7 @@ our (%init,
        [ 'REGEXP', 'SCALAR', 'SCALAR' ],
        [
          { 0 => $RE{weekday}, 1 => 'next', 2 => 'week' },
+         [],
          [ [ 0 ] ],
          [ '_weekday_next_week' ],
        ],
@@ -736,6 +847,7 @@ our (%init,
        [ 'REGEXP', 'SCALAR', 'SCALAR', 'SCALAR' ],
        [
          { 0 => $RE{day_enum}, 1 => 'day', 2 => 'next', 3 => 'month' },
+         [],
          [ [ 0 ] ],
          [ '_count_day_next_month' ],
        ],
@@ -744,6 +856,7 @@ our (%init,
        [ 'REGEXP', 'SCALAR', 'SCALAR', 'SCALAR' ],
        [
          { 0 => $RE{day_enum}, 1 => 'month', 2 => 'next', 3 => 'year' },
+         [],
          [ [ 0 ] ],
          [ '_count_month_next_year' ],
        ],
@@ -752,6 +865,7 @@ our (%init,
        [ 'SCALAR', 'REGEXP', 'SCALAR' ],
        [
          { 0 => 'in', 1 => $RE{number}, 2 => qr/minutes?/i },
+         [],
          [ [ 1 ] ],
          [ '_in_count_minutes' ],
        ]
@@ -760,6 +874,7 @@ our (%init,
        [ 'SCALAR', 'REGEXP', 'SCALAR' ],
        [
          { 0 => 'in', 1 => $RE{number}, 2 => 'hours' },
+         [],
          [ [ 1 ] ],
          [ '_in_count_hours' ],
        ],
@@ -768,6 +883,7 @@ our (%init,
        [ 'SCALAR', 'REGEXP', 'SCALAR' ],
        [
          { 0 => 'in', 1 => $RE{number}, 2 => qr/days?/i },
+         [],
          [ [ 1 ] ],
          [ '_in_count_days' ],
        ],
@@ -776,6 +892,7 @@ our (%init,
        [ 'REGEXP', 'SCALAR', 'SCALAR' ],
        [
          { 0 => $RE{weekday}, 1 => 'this', 2 => 'week' },
+         [],
          [ [ 0 ] ],
          [ '_weekday_this_week' ],
        ],
@@ -784,6 +901,7 @@ our (%init,
        [ 'REGEXP', 'REGEXP', 'SCALAR', 'REGEXP' ],
        [
          { 0 => $RE{day_enum}, 1 => $RE{weekday}, 2 => 'this', 3 => $RE{month} },
+         [],
          [ [ 0, 1, 3 ] ],
          [ '_count_weekday_this_month' ],
        ],
@@ -792,21 +910,25 @@ our (%init,
        [ 'REGEXP', 'REGEXP', 'SCALAR', 'SCALAR' ],
        [
          { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'before', 3 => 'yesterday' },
+         [],
          [ [ 0 ] ],
          [ '_daytime_variant_before_yesterday' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'before', 3 => 'tomorrow' },
+         [],
          [ [ 0 ] ],
          [ '_daytime_variant_before_tomorrow' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'after', 3 => 'yesterday' },
+         [],
          [ [ 0 ] ],
          [ '_daytime_variant_after_yesterday' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'after', 3 => 'tomorrow' },
+         [],
          [ [ 0 ] ],
          [ '_daytime_variant_after_tomorrow' ],
        ],
@@ -815,21 +937,25 @@ our (%init,
        [ 'REGEXP', 'REGEXP', 'SCALAR', 'SCALAR' ],
        [
          { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'before', 3 => 'noon' },
+         [],
          [ [ 0 ] ],
          [ '_hourtime_before_noon' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'after', 3 => 'noon' },
+         [],
          [ [ 0 ] ],
          [ '_hourtime_after_noon' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'before', 3 => 'midnight' },
+         [],
          [ [ 0 ] ],
          [ '_hourtime_before_midnight' ],
        ],
        [
          { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'after', 3 => 'midnight' },
+         [],
          [ [ 0 ] ],
          [ '_hourtime_after_midnight' ],
        ],
@@ -838,16 +964,19 @@ our (%init,
        [ 'SCALAR', 'SCALAR', 'REGEXP' ],
        [
          { 0 => 'yesterday', 1 => 'at', 2 => $RE{time_am} },
+         [],
          [ [], [ 2 ] ],
          [ '_day_yesterday', '_time' ],
        ],
        [
          { 0 => 'today', 1 => 'at', 2 => $RE{time_am} },
+         [],
          [ [], [ 2 ] ],
          [ '_day_today', '_time' ],
        ],
        [
          { 0 => 'tomorrow', 1 => 'at', 2 => $RE{time_am} },
+         [],
          [ [], [ 2 ] ],
          [ '_day_tomorrow', '_time' ],
        ],
@@ -856,6 +985,7 @@ our (%init,
        [ 'REGEXP', 'SCALAR', 'REGEXP' ],
        [
          { 0 => $RE{weekday}, 1 => 'at', 2 => $RE{time_am} },
+         [],
          [ [ 0 ], [ 2 ] ],
          [ '_weekday', '_time' ],
        ],
@@ -864,16 +994,19 @@ our (%init,
        [ 'SCALAR', 'SCALAR', 'REGEXP' ],
        [
          { 0 => 'yesterday', 1 => 'at', 2 => $RE{time_pm} },
+         [],
          [ [], [ 2 ] ],
          [ '_day_yesterday', '_at_pm' ],
        ],
        [
          { 0 => 'today', 1 => 'at', 2 => $RE{time_pm} },
+         [],
          [ [], [ 2 ] ],
          [ '_day_today', '_at_pm' ],
        ],
        [
          { 0 => 'tomorrow', 1 => 'at', 2 => $RE{time_pm} },
+         [],
          [ [], [ 2 ] ],
          [ '_day_tomorrow', '_at_pm' ],
        ],
@@ -882,6 +1015,7 @@ our (%init,
        [ 'REGEXP', 'SCALAR', 'REGEXP' ],
        [
          { 0 => $RE{weekday}, 1 => 'at', 2 => $RE{time_pm} },
+         [],
          [ [ 0 ], [ 2 ] ],
          [ '_weekday', '_at_pm' ],
        ],
@@ -890,11 +1024,13 @@ our (%init,
        [ 'REGEXP', 'REGEXP', 'REGEXP' ],
        [
          { 0 => $RE{monthday}, 1 => $RE{month}, 2 => $RE{year} },
+         [],
          [ [ 0, 1, 2 ] ],
          [ '_day_month_year' ],
        ],
        [
          { 0 => $RE{month}, 1 => $RE{monthday}, 2 => $RE{year} },
+         [],
          [ [ 1, 0, 2 ] ],
          [ '_day_month_year' ],
        ],
@@ -903,10 +1039,26 @@ our (%init,
        [ 'REGEXP', 'REGEXP', 'SCALAR', 'REGEXP' ],
        [
          { 0 => $RE{day_enum}, 1 => $RE{weekday}, 2 => 'in', 3 => $RE{month} },
+         [],
          [ [ 0, 1, 3 ] ],
          [ '_count_weekday_this_month' ],
        ],
     ],
+    count_weekday_from_now => [
+       [ 'REGEXP', 'REGEXP', 'SCALAR', 'SCALAR' ],
+       [
+         { 0 => $RE{number}, 1 => $RE{weekdays}, 2 => 'from', 3 => 'now' },
+         [ sub {
+             my $captured = shift;
+             return true if ($captured->{0} == 1 && $captured->{1} !~ /s$/);
+             return true if ($captured->{0} >  1 && $captured->{1} =~ /s$/);
+             return false;
+           }
+         ],
+         [ [ 0, 1 ] ],
+         [ '_count_weekday_from_now' ],
+       ],
+    ]
 );
 
 1;
@@ -952,7 +1104,7 @@ that the parser does not distinguish between lower/upper case):
  tomorrow evening
  march
  4th february
- november 3rd 
+ november 3rd
  saturday
  last wednesday
  last june
@@ -966,11 +1118,13 @@ that the parser does not distinguish between lower/upper case):
  this month
  6 am
  5am
+ 5:30am
  5am yesterday
  5am today
  5am tomorrow
  8 pm
  4pm
+ 4:20pm
  4pm yesterday
  4pm today
  4pm tomorrow
@@ -1068,6 +1222,8 @@ that the parser does not distinguish between lower/upper case):
  4th day last week
  3rd wednesday in november
  3rd month next year
+ 6 mondays from now
+ 1 friday from now
 
 =head2 Timespans
 
@@ -1088,6 +1244,7 @@ that the parser does not distinguish between lower/upper case):
  27/5/1979
  4:00
  17:00
+ 3:20:00
 
 =head1 SEE ALSO
 
